module Spoll
  class Client
    attr_reader :token, :version

    HEADERS = { 'Content-Type' => 'application/json; charset=utf-8' }

    def initialize(client_id, client_secret, host, version)
      @client = ::OAuth2::Client.new(client_id, client_secret, site: host)
      @version = version
      get_token
    end

    def post_match(params)
      post('/matches', { match: params })
    end

    def put_match(id, params)
      put("/matches/#{id}", { match: params })
    end

    def get_match(id, params = {})
      get("/matches/#{id}", params)
    end

    def get_events(params)
      get('/events', params)
    end

    def get_event(id, params = {})
      get("/events/#{id}", params)
    end

    private

    def get_token
      @token = @client.client_credentials.get_token
    end

    def get(url, params)
      request(:get, url, params, {})
    end

    def post(url, body)
      request(:post, url, {}, body)
    end

    def put(url, body)
      request(:put, url, {}, body)
    end

    def request(method, url, params, body)
      with_retry do
        json = token.send(method, "/#{version}/#{url}", headers: HEADERS, params: params, body: body.to_json)
        Spoll::Response.new(json)
      end
    end

    def with_retry
      tries = 0
      begin
        yield
      rescue ::OAuth2::Error => error
        if error.response.status.eql?(401)
          if tries < 3
            tries += 1
            get_token
            retry
          end
        else
          error.response
        end
      end
    end

  end
end
