module Spoll
  class Response
    attr_accessor :match, :matches,
                  :event, :events,
                  :event_type, :event_types,
                  :teams, :team


    def initialize(response)
      @response    = response
      @matches     = []
      @events      = []
      @event_types = []
      @teams       = []

      includes = [
        { model: Spoll::Models::Match, key: 'match', attribute:  'match' },
        { model: Spoll::Models::Match, key: 'matches', attribute:  'matches' },
        { model: Spoll::Models::Event, key: 'events', attribute:  'events' },
        { model: Spoll::Models::EventType, key: 'event_types', attribute:  'event_types' },
        { model: Spoll::Models::Team, key: 'teams', attribute:  'teams' }
      ]

      load_includes(includes)
      set_relations(includes)
    end

    def status
      @response.status
    end

    private

    def load_includes(includes)
      includes.each do |hash|
        load_include(hash[:model], hash[:key], hash[:attribute])
      end
    end

    def load_include(model, key, attribute)
      if @response.parsed.key?(key)
        parsed = @response.parsed[key]

        if parsed.is_a?(Array)
          load_array(attribute, parsed, model)
        else
          load_single(attribute, parsed, model)
        end
      end
    end

    def load_array(attribute, array, model)
      array.each do |record|
        send(attribute) << model.new(record)
      end
    end

    def load_single(attribute, value, model)
      self.send("#{attribute}=", model.new(value))
    end

    def set_relations(includes)
      includes.each do |hash|
        [*hash[:attribute]].each do |record|
          attribute = send(record)
          attribute.set_relations(self) if attribute.present?
        end
      end
    end

  end
end
